#!/usr/bin/env python
"""performance.py

Takes the pair of Score and Orchestra objects and creates the means to:
1) Output the combined .csd
2) Perform the .csd in real-time

I'm sure there is more that we could do later.

Right now, all this does is some basic text manipulation; we could potentially create more interesting objects later.

$Id: Performance.py 3 2006-05-05 19:12:58Z nknouf $
"""

import logging, os
from tempfile import mkstemp

logger = logging.getLogger("csound5Lib")

class Performance(object):
    """Encapsulates the entire performance"""

    def __init__(self, orchestra = None, score = None, options = '-d -m231 -H0 -R -W --expression-opt -+rtaudio=alsa -o dac:plughw:0', filename = None, csoundCommand = "/usr/local/bin/csound"):

        self.orchestra = orchestra
        self.score = score
        self.options = options
        self.filename = filename
        self.csoundCommand = csoundCommand

        self.csd = None

    def generateCSD(self):
        """Generate the csd file."""

        csd = ""

        csd = csd + "<CsoundSynthesizer>\n"

        # setup the options
        csd = csd + "<CsOptions>\n"
        csd = csd + self.options + "\n"
        csd = csd + "</CsOptions>\n"

        # setup the orchestra
        if not self.orchestra:
            logger.error("No orchestra given!  Returning with an error...")
            return 1
        else:
            csd = csd + self.orchestra.generateOrchestra()

        csd = csd + "\n"

        # setup the score 
        if not self.score:
            logger.error("No score given!  Returning with an error...")
            return 1
        else:
            csd = csd + self.score.generateScore()

        csd = csd + "\n"
        
        csd = csd + "</CsoundSynthesizer>"

        self.csd = csd
        return csd

    def render(self):
        """Render the piece in real-time using the options in the CSD file."""

        if self.filename is None:
            filename = mkstemp(".csd")[1]
            print filename
        else:
            filename = self.filename

        if not self.csd:
            logger.error("No csd file created!  Returning with an error...")
            return 1

        fp = file(filename, 'w')
        fp.write(self.csd)
        fp.close()

        #command = '%s -+rtaudio=coreaudio -o dac %s' % (self.csoundCommand, filename)
        command = '%s %s %s' % (self.csoundCommand, self.options, filename)
        #command = '%s %s' % (self.csoundCommand, filename)
        logger.info("Starting performance with command %s" % command)
        os.popen(command)        
