#!/usr/bin/env python

class Score(object):
    """Encapsulates the score"""

    def __init__(self, ftableList = None, noteList = None, name = '', author = '', comment = ''):

        self.name = name
        self.author = author
        self.comment = comment

        if ftableList is not None:
            self.ftableList = ftableList
        else:
            self.ftableList = []

        if noteList is not None:
            self.noteList = noteList
        else:
            self.noteList = []

    def addFtable(self, ftable):
        """Add a ftable to our list."""

        if not ftable in self.ftableList:
            self.ftableList.append(ftable)

    def addNote(self, note):
        """Add a note to our list."""

        self.noteList.append(note)


    def __generatePreamble(self):
        """Generate the preamble for the score."""

        # create the comments section
        preamble = self.comment + "\n"
        preamble = preamble + "; %s\n" % self.name
        preamble = preamble + "; %s\n" % self.author

        self.preamble = preamble
        return preamble

    def __generateFtables(self):
        """Generate the ftable list."""

        ftables = ""
        for ftable in self.ftableList:
            ftables = ftables + ftable.definition + "\n"

        self.ftables = ftables

        return ftables

    def __generateNotes(self):
        """Generate the note list."""

        notes = ""
        for note in self.noteList:
            notes = notes + note.definition + "\n"

        self.notes = notes 
        return notes 

    def generateScore(self):
        """Generate the score section."""

        score = "<CsScore>\n"

        score = score + self.__generatePreamble()
        score = score + "\n\n"

        score = score + "; ftables\n"
        score = score + self.__generateFtables()
        score = score + "\n\n"
        
        score = score + "; Notes\n"
        score = score + self.__generateNotes()

        score = score + "e\n</CsScore>"

        self.score = score

        return score

class Note(object):
    """A single object."""

    def __init__(self, number = None, definition = ''):
        if number is not None:
            self.number = number

        self.definition = definition

    def setDefinition(self, definition):
        if definition:
            logging.warn("Note definition already set, overwritting...")

        self.definition = definition

class Ftable(object):
    """A single ftable object."""

    def __init__(self, definition = ''):

        self.definition = definition

    def setDefinition(self, definition):
        if definition:
            logging.warn("Ftable definition already set, overwritting...")

        self.definition = definition
