import logging, random, select, socket, sys, threading, time, traceback, ConfigParser

import OSC

import csound5Lib
from csound5Lib import Orchestra, Score, Performance

csound5Lib.startLogging()
logger = logging.getLogger("csound5Lib")

# create our orchestra
orc = Orchestra.Orchestra(ksmps = 100)
instr1 = Orchestra.Instrument(definition = """
    instr 1
iNumber = p4
OSCsend 1, "localhost", 48001, "/number", "i", 1
    endin
""")
instr2 = Orchestra.Instrument(definition = """
; OSC receive
    instr 2
k1 init 0
k2 init 0
k3 init 0
k4 init 0
k5 init 0
k6 init 0
sjunk init 0
nextMsg:
    kk OSClisten giosc, "/variations2", "sffffff", sjunk, k1, k2, k3, k4, k5, k6
if (kk == 0) goto ex
    printk 0, k1
    printk 0, k2
    kgoto nextMsg
ex:
    endin
""")
instr107 = Orchestra.Instrument(definition = """
        instr   107
iamp = ampdb(p4)
inote = p7
istop = p6

OSCsend 1, "localhost", 47110, "/variations2", "sif", "note", inote, istop
if (istop == 0) igoto ex

a1      oscil   iamp, p5, p6
        outs     a1, a1
ex:
garvb = garvb + (a1 * 0.2)
        endin
""")

instr101 = Orchestra.Instrument(definition = """
    instr 101

    iNote = p4
    OSCsend 1, "localhost", 47110, "/variations2", "sif", "perform", iNote, p5
    endin""")


instr201 = Orchestra.Instrument(definition = """
   instr 201

iNote   = p19
iTimbre = p20
    OSCsend 1, "localhost", 47110, "/variations2", "sif", "stop", iNote, p4
if (p4 == 0) igoto finish

idisp = p6

icondfn = p7

imasf = p8

ivelf = p9

ispgmtrxf = p10

icntrf = p11

idampf = p12

aIn init 0


    kspgstf line p13, p3, p14

    kmasf   line p15, p3, p16

    kcntr   line p17, p3, p18

    kEnv    linseg 0, p3*0.09, 1, p3*0.01, 0.9, p3 - p3*(0.09 + 0.01 + 0.1), 0

;         icondfn, scnrat,  ivelf, imasf, ispgmtrxf, icntrf, idampf, kmasf, kspgstf, kcntr, kdamp, ilplkpos, irplkpos, kpos, kstrngth, ain, idisp, id

   ; scanu  icondfn,    .05,  ivelf, imasf, ispgmtrxf, icntrf, idampf, kmasf, kspgstf, kcntr,  -.05,      .1,        .5,   .5,       0,   aIn,  idisp,  1
   scanu  icondfn,    1/(20 * iTimbre),  ivelf, imasf, ispgmtrxf, icntrf, idampf, kmasf, kspgstf, kcntr,  -.05,      .1,        .5,   .5,       0,   aIn,  idisp,  1

;  scans         kamp,      kfreq, itrj, iid

a1 scans  ampdb(p4), p5,    7,   1, 1

aIn = (a1 / 3000) * 0.002

a1 dcblock a1

a1      butterlp    a1, p5 * 8
;a1      butterlp    a1, p5 * 8
a1  = a1 * kEnv

    outs    a1, a1
    garvb = garvb + (a1 * 0.2)

finish:
    endin
""")

gvar1 = Orchestra.Global(globalVar = """
garvb init 0""")
instr199 = Orchestra.Instrument(definition = """
        instr  199
        denorm garvb
aL, aR freeverb garvb, garvb , 0.3, 0.25, 44100, 0
        outs garvb + aL, garvb + aR
garvb = 0
        endin
""")

orc.addInstrument(instr107)
orc.addInstrument(instr101)
orc.addInstrument(instr201)
orc.addInstrument(instr199)
orc.addGlobal(gvar1)

# create a basic score
#sco = Score.Score()
f0 = Score.Ftable(definition = """f0 30""")
f1 = Score.Ftable(definition = """f1  0 4096 10   1""")
f111 = Score.Ftable(definition = """f111   0 128 10 1 0 0 0 1 0 0 0 1""")
f2 = Score.Ftable(definition = """;f2  0 4096 10   1  .5 .333 .25 .2 .166 .142 .125 .111 .1 .09 .083 .076 .071 .066 .062
f2     0 128 -7  1 128 1""")
f7 = Score.Ftable(definition = """; Trajectory
f7 0 128 -7 0 128 128""")

#f333 = Score.Ftable(definition = """; Spring matrix
#f333   0 16385 -23 "/media/nomadicsounds/Users/nknouf/Documents/Research/Courses/MIT/MAS.826J/variations10b/trunk/torus-128,8" """)
# Remember to provide an actual path!
f333 = Score.Ftable(definition = """; Spring matrix
f333   0 16385 -23 "%s" """)

f444 = Score.Ftable(definition = """; Centering Force
f444  0 128 -7 0 64 12 64 0""")

f5 = Score.Ftable(definition = """; Damping
f5 0 128 -7 1 128 1""")
f55 = Score.Ftable(definition = """f55 0 128 -7 .01 64 1 64 .01""")
f555 = Score.Ftable(definition = """f555 0 128 -7 1 128 0.0001""")

f6 = Score.Ftable(definition = """; Initial velocity
f6    0 128 -7 0 128 0""")
f666 = Score.Ftable(definition = """f666  0 128 -7 .02 128 0""")
f6666 = Score.Ftable(definition = """f6666 0 128 -7 .01 128 .01""")


#sco.addNote(Score.Note(definition = """i1 0 1 1"""))
#sco.addNote(Score.Note(definition = """i2 0 600"""))
#for i in range(2,10):
#    sco.addNote(Score.Note(definition = """i1 . 1 %d""" % i))

class variationsIIReceiver(object):
    """Setup a non-blocking UDP client to collect my graphical/physical paramters."""

    manager = OSC.CallbackManager()

    def __init__(self, host = '127.0.0.1', port = 57140, configFileName = "variations10bConfig.ini"):
        self.listenerSocket = socket.socket(socket.AF_INET, socket.SOCK_DGRAM)
        self.listenerSocket.bind((host, port))
        self.listenerSocket.setblocking(0)

        self.config = ConfigParser.ConfigParser()
        self.config.read(configFileName)

        # TODO
        # Add some actual error checking here
        level = self.config.get("general", "loggingLevel")
        logger.setLevel(getattr(logging, level))

        self.manager.add(self.parseData, "/variations2")
        #self.manager.add(self.startParsing, "/beginScore")
        #self.manager.add(self.stopParsing, "/endScore")
        self.orc = orc

        logger.debug("Initialized collection server")

    def run(self):
        """Run the server."""
        loopCounter = 0
        loopUpdatePeriod = 100

        logger.debug("Started server...")

        try:
            while 1:
                readers, writers, errors = select.select([self.listenerSocket], [], [], 1)
                #readers = False

                if readers and not (loopCounter % loopUpdatePeriod):
                    data = readers[0].recv(1024)
                    #print data
                    try:
                        self.manager.handle(data)
                    except:
                        logger.warning("Malformed OSC message")
                        pass

                loopCounter += 1
        except KeyboardInterrupt:
            logger.info("Got a keyboard interrupt, quitting...")
            self.listenerSocket.close()
            print self.sco.generateScore()
            sys.exit(0)
        except select.error, (num, description):
            logger.error("Got select error: %d, %s" % (num, description))
        except:
            print sys.exc_info()[0]
            print sys.exc_info()[1]
            print traceback.format_tb(sys.exc_info()[2])

    def startParsing(self, message, source):
        """Start the parsing process by creating an empty score."""
        self.logger.debug("Starting to create the score.")
        #print message
        #print source

    def parseData(self, message, source):
        #print message
        if (message[2] == 'start'):
            logger.debug("Setting up the score")
            self.sco = Score.Score()
            self.sco.addFtable(f0)
            self.sco.addFtable(f1)
            self.sco.addFtable(f111)
            self.sco.addFtable(f2)
            self.sco.addFtable(f7)
            scanTablePath = self.config.get("performance", "scanTablePath")
            f333.definition = f333.definition % str(scanTablePath)
            self.sco.addFtable(f333)
            self.sco.addFtable(f444)
            self.sco.addFtable(f5)
            self.sco.addFtable(f55)
            self.sco.addFtable(f555)
            self.sco.addFtable(f6)
            self.sco.addFtable(f666)
            self.sco.addFtable(f6666)
            self.sco.addNote(Score.Note(definition = "i199 0 30"))
            self.sco.addNote(Score.Note(definition = "i101 0.1 0.1 0 1.0 1"))
        elif (message[2] =='perform'):
            self.sco.addNote(Score.Note(definition = "i101 29.9 0.01 0 0.0 0"))
            self.perform()
        else:
            occurrence = int(message[6])
            #print occurrence

            timbre = message[7]
            #timbre = int(message[7])
            if (timbre == 1):
                dampFun = 5
            elif (timbre == 2):
                dampFun = 55
            elif (timbre == 3):
                dampFun = 555
            else:
                dampFun = 55
                #logger.warning("Timbre value (%f) was not expected..." % timbre)

            for event in range(0, occurrence):            
                if event == 0:
                    delta = 0
                else:
                    delta = 2 * random.random()

                self.sco.addNote(Score.Note(definition =  \
                # ; start dur amp freq disp invol
                """i201 %f %f %f %f 0 111 2 6 333 444 %d 0.2 0.2 2 2 0.1 0.3 %d %f""" % \
                (message[8] + delta,
                message[5],
                message[3],
                message[4],
                dampFun,
                int(message[2]),
                timbre)))
                
                self.sco.addNote(Score.Note(definition =  \
                """i201 %f 0.1 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 %d""" % \
                (message[8] + message[5] + delta, int(message[2]))))
                
            #self.sco.addNote(Score.Note(definition = """i107 %f %f %f %f %f %d""" % (message[8], message[5], message[3], message[4], message[7], int(message[2]))))        
            #self.sco.addNote(Score.Note(definition = """i107 %f 0.01 0 0 0 %d""" % (message[8] + message[5], int(message[2]))))

    def perform(self):
        """Stop the parsing process and render the score."""
        logger.debug("Starting the performance...")
        self.performance = Performance.Performance(orchestra = self.orc,
                                                score = self.sco, csoundCommand = self.config.get("csound", "path"), options = self.config.get("csound", "options"))

        self.performance.generateCSD()
        
        # write the score file
        fp = open('scores/%s.csd' % time.strftime('%Y%m%d%H%M%S'), 'w')
        fp.write(self.performance.csd)
        fp.close()

        self.performance.render()

class collectVariationsData(threading.Thread):
    """Setup a non-blocking UDP server to collect my graphical/physical paramters."""

    dataQueueLock = threading.Lock()
    manager = OSC.CallbackManager()

    def __init__(self, dataHandler = None, host = '127.0.0.1', port = 57140):
        threading.Thread.__init__(self)

        self.listenerSocket = socket.socket(socket.AF_INET, socket.SOCK_DGRAM)
        self.listenerSocket.bind((host, port))
        self.listenerSocket.setblocking(0)

        self.manager.add(self.startParsing, "/start")
        self.manager.add(self.parseData, "/variations2")
        self.manager.add(self.stopParsing, "/stop")

        logger.debug("Initialized collection server")

    def startParsing(self, message, source):
        """Start the parsing process by creating an empty score."""
        logger.debug("got start message")
        self.sco = Score.Score()

    def parseData(self, message, source):
        self.dataQueueLock.acquire()
        #print message
        self.dataQueueLock.release()

    def stopParsing(self, message, source):
        """Stop the parsing process and render the score."""

        logger.debug("got stop message")
        self.performance = Performance.Performance(orchestra = self.orc,
                                                score = self.sco)

        self.performance.generateCSD()

        self.performance.render()

    def run(self):
        """Run the server thread."""
        loopCounter = 0
        loopUpdatePeriod = 100

        logger.debug("Started server thread")

        try:
            while 1:
                readers, writers, errors = select.select([self.listenerSocket], [], [], 1)
                #readers = False

                if readers and not (loopCounter % loopUpdatePeriod):
                    data = readers[0].recv(1024)
                    try:
                        self.manager.handle(data)
                    except:
                        logger.warning("Malformed OSC message")
                        pass

                loopCounter += 1
        except KeyboardInterrupt:
            print "Got a keyboard interrupt, quitting..."
            sys.exit(0)
        #except select.error, (num, description):
        #    logger.error("Got select error: %d, %s" % (num, description))
        #except:
        #    print sys.exc_info()[0]
        #    print sys.exc_info()[1]
        #    print traceback.format_tb(sys.exc_info()[2])

    def close(self):
        self.listenerSocket.close()

if __name__ == "__main__":
    #p = Performance.Performance(orchestra = orc, score = sco)
    #p.generateCSD()
    #p.render()
    recv = variationsIIReceiver()
    recv.run()
